/* xdwapi sample 11

Function :
	Attatches a signature to a DocuWorks file.
How to use :
	xdwsmpl11 input_file output_file signature_module
		signature_module : DocuWorks Electronic Signature=0 Electronic Certificate=1
*/

#include <stdio.h>
#include <windows.h>
#include <xdw_api.h>

void print_error(int code);
void print_module_error(int module, int code);

// Print usage.
void print_usage()
{
	fprintf(stderr, "Usage : xdwsmpl11 input_file output_file signature_module\n");
	fprintf(stderr, "\tsignature_module : DocuWorks Electronic Signature=0 Digital Certificate=1\n");
}


int main(int argc, char** argv)
{
	if (argc != 4) {
		print_usage();
		return 0;
	}
	int sig_type = atoi(argv[3]);
	if (sig_type != 0 && sig_type != 1) {
		print_usage();
		return 0;
	}

	XDW_SIGNATURE_OPTION_V5 option;
	option.nSize = sizeof(XDW_SIGNATURE_OPTION_V5);
	option.nPage = 1; // first page
	option.nHorPos = 1000; // 10mm from the left
	option.nVerPos = 1000; // 10mm from the top
	if (sig_type == 0) {
		option.nSignatureType = XDW_SIGNATURE_STAMP;
	} else {
		option.nSignatureType = XDW_SIGNATURE_PKI;
	}

	// Sign.
	XDW_SIGNATURE_MODULE_STATUS module_status;
	module_status.nSize = sizeof(XDW_SIGNATURE_MODULE_STATUS);
	int api_result = XDW_SignDocument(argv[1], argv[2], &option, NULL, NULL, &module_status);

	if (api_result < 0) {
		// Print XDWAPI error.
		print_error(api_result);
		if (api_result == XDW_E_SIGNATURE_MODULE) {
			// Print error unique to the signature module.
			print_module_error(module_status.nSignatureType, module_status.nErrorStatus);
		}
		return 0;
	}

	return 1;
}

// Print XDWAPI error.
void print_error(int code)
{
	switch (code) {
	case XDW_E_NOT_INSTALLED:
		fprintf(stderr, "DocuWorks not installed.\n");
		break;
	case XDW_E_FILE_NOT_FOUND:
		fprintf(stderr, "File not found.\n");
		break;
	case XDW_E_ACCESSDENIED:
	case XDW_E_INVALID_NAME:
	case XDW_E_BAD_NETPATH:
		fprintf(stderr, "File not opened.\n");
		break;
	case XDW_E_BAD_FORMAT:
		fprintf(stderr, "File bad format.\n");
		break;
	case XDW_E_INVALID_ACCESS:
		fprintf(stderr, "Invalid access.\n");
		break;
	case XDW_E_SIGNATURE_MODULE:
		fprintf(stderr, "Error unique to the signature module occurred.\n");
		break;
	default:
		fprintf(stderr, "Error.\n");
		break;
	}
}

// Print error unique to the signature module.
void print_module_error(int module, int code)
{
	switch (module) {
	// DocuWorks Electronic Signature
	case XDW_SIGNATURE_STAMP:
		fprintf(stderr, "DocuWorks Electronic Signature : ");
		switch (code) {
		case XDW_SIGNATURE_STAMP_ERROR_NO_OPENING_CASE:
			fprintf(stderr, "The processing failed because no open Electronic Signature Case exists.\n");
			break;
		case XDW_SIGNATURE_STAMP_ERROR_NO_SELFSTAMP:
			fprintf(stderr, "My Electronic Signature is not available in the Electronic Signature Case.\n");
			break;
		case XDW_SIGNATURE_STAMP_ERROR_OUT_OF_VALIDITY:
			fprintf(stderr, "The Electronic Signature is outside the valid date.\n");
			break;
		case XDW_SIGNATURE_STAMP_ERROR_INVALID_DATA:
			fprintf(stderr, "The processing failed because of uncomformity of data such as document to attach the signature and Electronic Signature Case.\n");
			break;
		case XDW_SIGNATURE_STAMP_ERROR_OUT_OF_MEMORY:
			fprintf(stderr, "The processing failed due to insufficient memory.\n");
			break;
		case XDW_SIGNATURE_STAMP_ERROR_UNKNOWN:
			fprintf(stderr, "The processing failed due to unexpected error.\n");
			break;
		case XDW_SIGNATURE_STAMP_ERROR_OK:
			fprintf(stderr, "The processing succeeded.\n");
			break;
		default:
			fprintf(stderr, "Error.\n");
			break;
		}
		break;
	// Electronic Certificate
	case XDW_SIGNATURE_PKI:
		fprintf(stderr, "Electronic Certificate module : ");
		switch (code) {
		case XDW_SIGNATURE_PKI_ERROR_BAD_PLATFORM:
			fprintf(stderr, "Executing environment is inappropriate.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_WRITE_REG_ERROR:
			fprintf(stderr, "Writing operation to the registry failed.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_BAD_TRUST_LEVEL:
			fprintf(stderr, "The set trustworthiness level is invalid.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_BAD_REVOKE_CHECK_TYPE:
			fprintf(stderr, "The set verification method is invalid.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_BAD_AUTO_IMPORT_CERT_FLAG:
			fprintf(stderr, "The value of auto registration flag of the Certificate is invalid.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_BAD_SIGN_CONFIG:
			fprintf(stderr, "The setting of the signature is invalid.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_NO_IMAGE_FILE:
			fprintf(stderr, "Image file does not exist.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_BAD_SIGN_CERT:
			fprintf(stderr, "The selected Certificate can no longer be used.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_NO_SIGN_CERT:
			fprintf(stderr, "The Certificate for signature is not selected.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_NOT_USE_PRIVATE_KEY:
			fprintf(stderr, "Access to Private key failed.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_INVALID:
			fprintf(stderr, "The Certificate is already revoked.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_BAD_SIGN:
			fprintf(stderr, "The signature of the Certificate is invalid.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_REVOKE_CHECK_ERROR:
			fprintf(stderr, "Revocation of the Certificate cannot be confirmed.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_OUT_OF_VALIDITY:
			fprintf(stderr, "The Certificate is outside the valid date.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_NO_CERT:
			fprintf(stderr, "The Certificate does not exist.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_FAILURE_IMPOPT_CERT:
			fprintf(stderr, "Registration of the Certificate failed.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_NO_ROOT_CERT:
			fprintf(stderr, "The Root Certificate does not exist.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_BAD_CERT_SIZE:
			fprintf(stderr, "The Certificate size is invalid.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_BAD_ARG:
			fprintf(stderr, "The argument is invalid.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_UNKNOWN:
			fprintf(stderr, "Unknown error.\n");
			break;
		case XDW_SIGNATURE_PKI_ERROR_OK:
			fprintf(stderr, "The processing succeeded.\n");
			break;
		default:
			fprintf(stderr, "Error.\n");
			break;
		}
		break;
	default:
		fprintf(stderr, "Unknown signature module.\n");
		break;
	}
}
